# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 03.09.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 1, task 4.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation

import os

import cv2
import numpy as np
from scipy import signal # pylint: disable=import-error
import matplotlib.pyplot as plt

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def main():
  """Main"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 1')
  print('Task 4')
  print('-'*80)

  img_filename = '../../images/football.jpg' # Change this to suit you
  image = cv2.imread(img_filename, cv2.IMREAD_GRAYSCALE)

  # Cast the image to float (gradients can be negative)
  image = image.astype(np.float32)

  # Compute vertical gradient (poor-mans approximation)
  f_x = np.array([[-1, -2, -1],
                  [0, 0, 0],
                  [1, 2, 1]])
  grad_x = signal.convolve2d(image, f_x, mode='same', boundary='symm')

  # Compute horizontal gradient (poor-mans approximation)
  f_y = np.array([[-1, 0, 1],
                  [-2, 0, 2],
                  [-1, 0, 1]])
  grad_y = signal.convolve2d(image, f_y, mode='same', boundary='symm')

  # Compute gradient magnitude
  grad_magn = np.sqrt(grad_x**2 + grad_y**2)

  # Put it in [0, 255] value range
  grad_magn = 255*(grad_magn - np.min(grad_magn)) / (np.max(grad_magn) - np.min(grad_magn))

  # Manually threshold the gradient magnitude image, and use it to retrive the seam
  thresh1 = 100
  thresholded_1 = (grad_magn < thresh1)*0 + (grad_magn >= thresh1)*255
  mask_1 = (thresholded_1 == 255)*1
  masked_1 = mask_1 * image

  # Manually threshold the original image, and use it to retrive the seam
  thresh2 = 225
  thresholded_2 = (image < thresh2)*0 + (image >= thresh2)*255
  mask_2 = (thresholded_2 == 255)*1
  masked_2 = mask_2 * image


  # Plot the results
  write_dir = 'results/images'
  fig_num = plot_image(image, fig_num=0,
                       write_file=os.path.join(write_dir, 'football.png'))
  fig_num = plot_image(grad_x, fig_num,
                       write_file=os.path.join(write_dir, 'football_grad_x.png'))
  fig_num = plot_image(grad_y, fig_num,
                       write_file=os.path.join(write_dir, 'football_grad_y.png'))
  fig_num = plot_image(grad_magn, fig_num,
                       write_file=os.path.join(write_dir, 'football_grad_magn.png'))
  fig_num = plot_image(thresholded_1, fig_num,
                       write_file=os.path.join(write_dir, 'football_thresh_grad.png'))
  fig_num = plot_image(masked_1, fig_num,
                       write_file=os.path.join(write_dir, 'football_seam_outline.png'))
  fig_num = plot_image(thresholded_2, fig_num,
                       write_file=os.path.join(write_dir, 'football_thresh.png'))
  fig_num = plot_image(masked_2, fig_num,
                       write_file=os.path.join(write_dir, 'football_seam.png'))

  fig = plt.figure(0)

  images = [image, grad_x, grad_y, grad_magn, thresholded_1, masked_1,
            thresholded_2, masked_2]
  titles = ['Original', 'Vertical gradient', 'Horizontal gradient',
            'Gradient magnitude', 'Thresholded gradient magnitude',
            'Seam outline', 'Thresholded original image', 'Seam']

  for i in range(8):
    plt.subplot(4, 2, i+1), plt.imshow(images[i], 'gray')
    plt.title(titles[i]), plt.xticks([]), plt.yticks([])
  fig.savefig('results/images/football_segmentations.png', bbox_inches='tight',
              pad_inches=0)


  plt.show()

if __name__ == "__main__":
  main()
